@@ -13,36 +13,45 @@
 
    Curl maintains some minor information in process-global variables.
    One must call curl_global_init() to initialize them before calling
-   any other Curl library function.
+   any other Curl library function.  This is not state information --
+   it is constants.  They just aren't the kind of constants that the
+   library loader knows how to set, so there has to be this explicit
+   call to set them up.  The matching function curl_global_cleanup()
+   returns resources these use (to wit, the constants live in
+   malloc'ed storage and curl_global_cleanup() frees the storage).
 
-   But note that one of those global variables tells that
-   curl_global_init() has been called, and if you call it again,
-   it's just a no-op.  So we can call it for each of many transport
-   object constructors.
+   So our setup_global_const transport operation calls
+   curl_global_init() and our teardown_global_const calls
+   curl_global_cleanup().
 
-   The actual global variables appear to be constant so that it's OK
-   for them to be shared among multiple objects and threads.  And we never
-   want other than the defaults.  They're things like the identity of
-   the malloc routine.
+   The Curl library is supposed to maintain a reference count for the
+   global constants so that multiple modules using the library and
+   independently calling curl_global_init() and curl_global_cleanup()
+   are not a problem.  But today, it just keeps a flag "I have been
+   initialized" and the first call to curl_global_cleanup() destroys
+   the constants for everybody.  Therefore, the user of the Xmlrpc-c
+   Curl client XML transport must make sure not to call
+   teardownGlobalConstants until everything else in his program is
+   done using the Curl library.
 
-   Note that not only could many Xmlrpc-c Curl XML transport
-   objects be using Curl in the same process, but the Xmlrpc-c user may
-   have uses of his own.  So this whole fragile thing works only as long
-   as the user doesn't need to set these global variables to something
-   different from the defaults (because those are what we use).
+   Note that curl_global_init() is not threadsafe (with or without the
+   reference count), therefore our setup_global_const is not, and must
+   be called when no other thread in the process is running.
+   Typically, one calls it right at the beginning of the program.
 
-   curl_global_cleanup() reverts the process back to a state in which
-   nobody can use the Curl library.  So we can't ever call it.  This
-   means there will be some memory leakage, but since curl_global_init
-   only ever has effect once, the amount of leakage is trivial.  The
-   user can do his own curl_global_cleanup() if he really cares.
+   There are actually two other classes of global variables in the
+   Curl library, which we are ignoring: debug options and custom
+   memory allocator function identities.  Our code never changes these
+   global variables from default.  If something else in the user's
+   program does, User is responsible for making sure it doesn't
+   interfere with our use of the library.
 
-   Some of the Curl global variables are actually the SSL library global
-   variables.  The SSL library has the same disease.
+   Note that when we say what the Curl library does, we're also
+   talking about various other libraries Curl uses internally, and in
+   fact much of what we're saying about global variables springs from
+   such subordinate libraries as OpenSSL and Winsock.
 -----------------------------------------------------------------------------*/
 
-
-
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -59,8 +68,10 @@
 
 #include "xmlrpc-c/base.h"
 #include "xmlrpc-c/base_int.h"
+#include "xmlrpc-c/string_int.h"
 #include "xmlrpc-c/client.h"
 #include "xmlrpc-c/client_int.h"
+#include "xmlrpc-c/transport.h"
 #include "version.h"
 
 #include <curl/curl.h>
@@ -613,6 +624,11 @@ initWindowsStuff(xmlrpc_env * const envP ATTR_UNUSED) {
 
 #if defined (WIN32)
     /* This is CRITICAL so that cURL-Win32 works properly! */
+    
+    /* So this commenter says, but I wonder why.  libcurl should do the
+       required WSAStartup() itself, and it looks to me like it does.
+       -Bryan 06.01.01
+    */
     WORD wVersionRequested;
     WSADATA wsaData;
     int err;
@@ -639,6 +655,16 @@ initWindowsStuff(xmlrpc_env * const envP ATTR_UNUSED) {
 
 
 
+static void
+termWindowsStuff(void) {
+
+#if defined (WIN32)
+    WSACleanup();
+#endif
+}
+
+
+
 static void
 getXportParms(xmlrpc_env *  const envP ATTR_UNUSED,
               const struct xmlrpc_curl_xportparms * const curlXportParmsP,
@@ -815,6 +841,8 @@ freeXportParms(const struct xmlrpc_client_transport * const transportP) {
         xmlrpc_strfree(curlSetupP->sslKeyType);
     if (curlSetupP->sslKey)
         xmlrpc_strfree(curlSetupP->sslKey);
+    if (curlSetupP->sslCertPasswd)
+        xmlrpc_strfree(curlSetupP->sslCertPasswd);
     if (curlSetupP->sslCertType)
         xmlrpc_strfree(curlSetupP->sslCertType);
     if (curlSetupP->sslCert)
@@ -942,14 +970,6 @@ create(xmlrpc_env *                      const envP,
 
     assertConstantsMatch();
 
-    initWindowsStuff(envP);
-
-    /* Here is the global variable sadness that makes it impossible for
-       a transport object to be truly modular.  See the explanation of
-       Curl global variables at the top of this file.
-    */
-    curl_global_init(CURL_GLOBAL_ALL);
-
     MALLOCVAR(transportP);
     if (transportP == NULL)
         xmlrpc_faultf(envP, "Unable to allocate transport descriptor.");
@@ -978,16 +998,6 @@ create(xmlrpc_env *                      const envP,
 
 
 
-static void
-termWindowStuff(void) {
-
-#if defined (WIN32)
-    WSACleanup();
-#endif
-}
-
-
-
 static void
 assertNoOutstandingCurlWork(struct curlMulti * const curlMultiP) {
 
@@ -1027,12 +1037,6 @@ destroy(struct xmlrpc_client_transport * const clientTransportP) {
 
     freeXportParms(clientTransportP);
 
-    /* We want to call curl_global_cleanup() now, but can't.  See
-       explanation of Curl global variables at the top of this file.
-    */
-
-    termWindowStuff();
-
     free(clientTransportP);
 }
 
@@ -1085,7 +1089,7 @@ addUserAgentHeader(xmlrpc_env *         const envP,
                         "User-Agent: %s Xmlrpc-c/%s Curl/%s",
                         userAgent, XMLRPC_C_VERSION, curlVersion);
         
-        if (userAgentHeader == NULL)
+        if (userAgentHeader == xmlrpc_strsol)
             xmlrpc_faultf(envP, "Couldn't allocate memory for "
                           "User-Agent header");
         else {
@@ -1109,7 +1113,7 @@ addAuthorizationHeader(xmlrpc_env *         const envP,
         xmlrpc_asprintf(&authorizationHeader, "Authorization: %s",
                         basicAuthInfo);
             
-        if (authorizationHeader == NULL)
+        if (authorizationHeader == xmlrpc_strsol)
             xmlrpc_faultf(envP, "Couldn't allocate memory for "
                           "Authorization header");
         else {
@@ -1164,70 +1168,71 @@ setupCurlSession(xmlrpc_env *             const envP,
 
     curl_easy_setopt(curlSessionP, CURLOPT_POST, 1);
     curl_easy_setopt(curlSessionP, CURLOPT_URL, curlTransactionP->serverUrl);
-    curl_easy_setopt(curlSessionP, CURLOPT_SSL_VERIFYPEER,
-                     curlSetupP->sslVerifyPeer);
-    curl_easy_setopt(curlSessionP, CURLOPT_SSL_VERIFYHOST,
-                     curlSetupP->sslVerifyHost ? 2 : 0);
-
-    if (curlSetupP->networkInterface)
-        curl_easy_setopt(curlSessionP, CURLOPT_INTERFACE,
-                         curlSetupP->networkInterface);
-    if (curlSetupP->sslCert)
-        curl_easy_setopt(curlSessionP, CURLOPT_SSLCERT,
-                         curlSetupP->sslCert);
-    if (curlSetupP->sslCertType)
-        curl_easy_setopt(curlSessionP, CURLOPT_SSLCERTTYPE,
-                         curlSetupP->sslCertType);
-    if (curlSetupP->sslCertPasswd)
-        curl_easy_setopt(curlSessionP, CURLOPT_SSLCERTPASSWD,
-                         curlSetupP->sslCertPasswd);
-    if (curlSetupP->sslKey)
-        curl_easy_setopt(curlSessionP, CURLOPT_SSLKEY,
-                         curlSetupP->sslKey);
-    if (curlSetupP->sslKeyType)
-        curl_easy_setopt(curlSessionP, CURLOPT_SSLKEYTYPE,
-                         curlSetupP->sslKeyType);
-    if (curlSetupP->sslKeyPasswd)
-        curl_easy_setopt(curlSessionP, CURLOPT_SSLKEYPASSWD,
-                         curlSetupP->sslKeyPasswd);
-    if (curlSetupP->sslEngine)
-        curl_easy_setopt(curlSessionP, CURLOPT_SSLENGINE,
-                         curlSetupP->sslEngine);
-    if (curlSetupP->sslEngineDefault)
-        curl_easy_setopt(curlSessionP, CURLOPT_SSLENGINE_DEFAULT);
-    if (curlSetupP->sslVersion != XMLRPC_SSLVERSION_DEFAULT)
-        curl_easy_setopt(curlSessionP, CURLOPT_SSLVERSION,
-                         curlSetupP->sslVersion);
-    if (curlSetupP->caInfo)
-        curl_easy_setopt(curlSessionP, CURLOPT_CAINFO,
-                         curlSetupP->caInfo);
-    if (curlSetupP->caPath)
-        curl_easy_setopt(curlSessionP, CURLOPT_CAPATH,
-                         curlSetupP->caPath);
-    if (curlSetupP->randomFile)
-        curl_easy_setopt(curlSessionP, CURLOPT_RANDOM_FILE,
-                         curlSetupP->randomFile);
-    if (curlSetupP->egdSocket)
-        curl_easy_setopt(curlSessionP, CURLOPT_EGDSOCKET,
-                         curlSetupP->egdSocket);
-    if (curlSetupP->sslCipherList)
-        curl_easy_setopt(curlSessionP, CURLOPT_SSL_CIPHER_LIST,
-                         curlSetupP->sslCipherList);
 
     XMLRPC_MEMBLOCK_APPEND(char, envP, callXmlP, "\0", 1);
     if (!envP->fault_occurred) {
         curl_easy_setopt(curlSessionP, CURLOPT_POSTFIELDS, 
                          XMLRPC_MEMBLOCK_CONTENTS(char, callXmlP));
         
-        curl_easy_setopt(curlSessionP, CURLOPT_FILE, responseXmlP);
-        curl_easy_setopt(curlSessionP, CURLOPT_HEADER, 0 );
         curl_easy_setopt(curlSessionP, CURLOPT_WRITEFUNCTION, collect);
+        curl_easy_setopt(curlSessionP, CURLOPT_FILE, responseXmlP);
+        curl_easy_setopt(curlSessionP, CURLOPT_HEADER, 0);
         curl_easy_setopt(curlSessionP, CURLOPT_ERRORBUFFER, 
                          curlTransactionP->curlError);
         curl_easy_setopt(curlSessionP, CURLOPT_NOPROGRESS, 1);
         
         curl_easy_setopt(curlSessionP, CURLOPT_HTTPHEADER, 
                          curlTransactionP->headerList);
+
+        curl_easy_setopt(curlSessionP, CURLOPT_SSL_VERIFYPEER,
+                         curlSetupP->sslVerifyPeer);
+        curl_easy_setopt(curlSessionP, CURLOPT_SSL_VERIFYHOST,
+                         curlSetupP->sslVerifyHost ? 2 : 0);
+
+        if (curlSetupP->networkInterface)
+            curl_easy_setopt(curlSessionP, CURLOPT_INTERFACE,
+                             curlSetupP->networkInterface);
+        if (curlSetupP->sslCert)
+            curl_easy_setopt(curlSessionP, CURLOPT_SSLCERT,
+                             curlSetupP->sslCert);
+        if (curlSetupP->sslCertType)
+            curl_easy_setopt(curlSessionP, CURLOPT_SSLCERTTYPE,
+                             curlSetupP->sslCertType);
+        if (curlSetupP->sslCertPasswd)
+            curl_easy_setopt(curlSessionP, CURLOPT_SSLCERTPASSWD,
+                             curlSetupP->sslCertPasswd);
+        if (curlSetupP->sslKey)
+            curl_easy_setopt(curlSessionP, CURLOPT_SSLKEY,
+                             curlSetupP->sslKey);
+        if (curlSetupP->sslKeyType)
+            curl_easy_setopt(curlSessionP, CURLOPT_SSLKEYTYPE,
+                             curlSetupP->sslKeyType);
+        if (curlSetupP->sslKeyPasswd)
+            curl_easy_setopt(curlSessionP, CURLOPT_SSLKEYPASSWD,
+                             curlSetupP->sslKeyPasswd);
+        if (curlSetupP->sslEngine)
+            curl_easy_setopt(curlSessionP, CURLOPT_SSLENGINE,
+                             curlSetupP->sslEngine);
+        if (curlSetupP->sslEngineDefault)
+            curl_easy_setopt(curlSessionP, CURLOPT_SSLENGINE_DEFAULT);
+        if (curlSetupP->sslVersion != XMLRPC_SSLVERSION_DEFAULT)
+            curl_easy_setopt(curlSessionP, CURLOPT_SSLVERSION,
+                             curlSetupP->sslVersion);
+        if (curlSetupP->caInfo)
+            curl_easy_setopt(curlSessionP, CURLOPT_CAINFO,
+                             curlSetupP->caInfo);
+        if (curlSetupP->caPath)
+            curl_easy_setopt(curlSessionP, CURLOPT_CAPATH,
+                             curlSetupP->caPath);
+        if (curlSetupP->randomFile)
+            curl_easy_setopt(curlSessionP, CURLOPT_RANDOM_FILE,
+                             curlSetupP->randomFile);
+        if (curlSetupP->egdSocket)
+            curl_easy_setopt(curlSessionP, CURLOPT_EGDSOCKET,
+                             curlSetupP->egdSocket);
+        if (curlSetupP->sslCipherList)
+            curl_easy_setopt(curlSessionP, CURLOPT_SSL_CIPHER_LIST,
+                             curlSetupP->sslCipherList);
     }
 }
 
@@ -1807,7 +1812,42 @@ call(xmlrpc_env *                     const envP,
 
 
 
+static void
+setupGlobalConstants(xmlrpc_env * const envP) {
+/*----------------------------------------------------------------------------
+   See longwinded discussion of the global constant issue at the top of
+   this file.
+-----------------------------------------------------------------------------*/
+    initWindowsStuff(envP);
+
+    if (!envP->fault_occurred) {
+        CURLcode rc;
+
+        rc = curl_global_init(CURL_GLOBAL_ALL);
+        
+        if (rc != CURLE_OK)
+            xmlrpc_faultf(envP, "curl_global_init() failed with code %d", rc);
+    }
+}
+
+
+
+static void
+teardownGlobalConstants(void) {
+/*----------------------------------------------------------------------------
+   See longwinded discussionof the global constant issue at the top of
+   this file.
+-----------------------------------------------------------------------------*/
+    curl_global_cleanup();
+
+    termWindowsStuff();
+}
+
+
+
 struct xmlrpc_client_transport_ops xmlrpc_curl_transport_ops = {
+    &setupGlobalConstants,
+    &teardownGlobalConstants,
     &create,
     &destroy,
     &sendRequest,
